package assignment_matrix_layouter;

import util.*;

/**
 * Zeichenflche fr ASCII-Art.
 * 
 * @author Heiner Kcker
 */
public class AsciiCanvas
{
    /**
     * inneres Array mit einem StringBuffer je Zeile
     */
    private UnSynchStringBuffer[] lineBuffArr;

    private int lineWidth;
    
    /**
     * Constructor.
     *
     */
    public AsciiCanvas()
    {
        super();
        this.lineBuffArr = new UnSynchStringBuffer[ 0 ];
    }

    /**
     * Constructor.
     *
     * @param pLineCount Anzahl Zeilen
     */
    public AsciiCanvas(
            final int pLineCount,
            final int pLineWidth )
    {
        super();
        this.lineBuffArr = new UnSynchStringBuffer[ pLineCount ];
        this.lineWidth = pLineWidth;
        initBuffers();
    }
    
    /**
     * Initialisiere das Zeilen-Array mit String-Puffern.
     */
    private void initBuffers()
    {
        final String spaceStr = StringUtil.spaces( this.lineWidth );
        for (int i = 0; i < this.lineBuffArr.length; i++)
        {
            if ( this.lineBuffArr[ i ] == null )
            {
                final UnSynchStringBuffer lineBuff = new UnSynchStringBuffer( this.lineWidth );
                /*
                 * Anfangsbefllung
                 */
                lineBuff.append( spaceStr );
                
                this.lineBuffArr[ i ] = lineBuff;
            }
        }
    }

    /**
     * 
     * @param pRow
     * @param pCol
     * @param pChr
     */
    public void setCharAt(
            final int pRow,
            final int pCol,
            final char pChr)
    {
        this.expandLines( pRow + 1 );
        
        final UnSynchStringBuffer strBuff = this.lineBuffArr[ pRow ];
        strBuff.fillSpacesToLength( pCol + 1 );
        strBuff.setCharAt( pCol, pChr);
    }

    /**
     * Get char at psosition.
     * 
     * @param pRow
     * @param pCol
     * @return
     */
    public char charAt(
            final int pRow ,
            final int pCol )
    {
        if ( pRow >= this.lineBuffArr.length )
        {
            return '\0';
        }
        final UnSynchStringBuffer lineBuff = this.lineBuffArr[ pRow ];
        if ( pCol >= lineBuff.length() )
        {
            return '\0';
        }
        return lineBuff.charAt( pCol );
    }


    /**
     * Vergrssere das innere Array, wen ntig.
     * 
     * @param pLength
     */
    private void expandLines(
            final int pLength )
    {
        if ( this.lineBuffArr.length >= pLength )
        {
            return;
        }
        final UnSynchStringBuffer[] tmpLineBuffArr = new UnSynchStringBuffer[ pLength ];
        System.arraycopy( this.lineBuffArr, 0, tmpLineBuffArr, 0, this.lineBuffArr.length );
        this.lineBuffArr = tmpLineBuffArr;
        this.initBuffers();
    }
    
    /**
     * Get line.
     * 
     * @param pCol
     * @return
     */
    public UnSynchStringBuffer getLineBuff(
            final int pCol)
    {
        return this.lineBuffArr[ pCol ];
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return UnSynchStringBuffer.concat(
                this.lineBuffArr, 
                "\n" );
    }

    /**
     * @return
     */
    public int length()
    {
        return this.lineBuffArr.length;
    }

    /**
     * Get a clone.
     * 
     * @return
     */
    public AsciiCanvas getClone()
    {
        final AsciiCanvas clone = 
            new AsciiCanvas(
                    this.lineBuffArr.length,
                    this.lineWidth );
        for (int i = 0; i < this.lineBuffArr.length; i++)
        {
            final UnSynchStringBuffer strBuff = this.lineBuffArr[i];
            clone.lineBuffArr[ i ] = strBuff.getClone();
        }
        return clone;
    }
    
    /**
     * Zurckgeben einer geclonten ASCII-Leinwand 
     * mit laufenden Nummern .
     * 
     * @return
     */
    public Object addLineNr()
    {
        final AsciiCanvas clone = this.getClone();
        // Anzahl Stellen fr die Zeilen-Nummer
        final int digitCount = String.valueOf( clone.lineBuffArr.length ).length();
        
        for (int i = 0; i < clone.lineBuffArr.length; i++)
        {
            final String lfdNrStr = StringUtil.strzero( i , digitCount ) + ":";
            final UnSynchStringBuffer strBuff = clone.lineBuffArr[i];
            clone.lineBuffArr[ i ] = new UnSynchStringBuffer( lfdNrStr + strBuff );
        }
        return clone;
    }

}
