package assignment_matrix_layouter;

import java.util.*;

/**
 * @author Heiner Kcker
 */
public class AssignmentMatrix
{
    /**
     * Menge der {@link AssignmentMatrixNode}-Objekte,
     * welche diese {@link AssignmentMatrix}
     * enthlt.
     */
    private final TreeSet nodeSet = new TreeSet();
    
    /**
     * Map zum Finden des Knoten-Objektes zum Namen des Knoten.
     */
    private final HashMap nameToNodeMap = new HashMap();
    
    /**
     * Menge mit den Verbindungen {@link AssignmentMatrixConnectionVO}
     * in dieser Matrix.
     */
    private final List connVoSet = new ArrayList();
    
    /**
     * add node.
     * 
     * @param pNode Node to add
     */
    public void add(
            final AssignmentMatrixNode pNode )
    {
        if ( this.nodeSet.contains( pNode  ) )
        {
            throw new IllegalArgumentException( "node exists already in matrix: " + pNode );
        }
        
        // Rck-Referenz setzen
        pNode .matrix = this;
        
        this.nodeSet.add( pNode  );
        if ( this.nameToNodeMap.put( pNode .name , pNode  ) != null )
        {
            throw new IllegalArgumentException( "node name already in matrix: " + pNode );
        }
    }

    /**
     * Bestimmen des Maximum aller Zeilen-Nummern.
     * @return
     */
    public int getMaxLine()
    {
        int maxLineNr = -1;
        for (Iterator iter = this.nodeSet.iterator(); iter.hasNext();)
        {
            final AssignmentMatrixNode node = (AssignmentMatrixNode) iter.next();
            maxLineNr = Math.max( maxLineNr, node.lineNr );
        }
        return maxLineNr;
    }

    /**
     * @return
     */
    public int getNodeCount()
    {
        return this.nodeSet.size();
    }

    /**
     * @return
     */
    public Iterator nodeIterator()
    {
        return Collections.unmodifiableSet( this.nodeSet ).iterator();
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return this.getClass().getName() + " " + this.nodeSet;
    }

    /**
     * 
     * @param pI
     * @return
     */
    public AssignmentMatrixNode getNodeForLineNr(
            final int pLineNr )
    {
        // TODO wegen dieser uneffektiven Methode entweder umstellen auf Map oder eine zustzliche positionsbezogene List ( list.add( lineNr, node ) ) machen
        for (Iterator iter = this.nodeIterator(); iter.hasNext();)
        {
            final AssignmentMatrixNode node = (AssignmentMatrixNode) iter.next();
            if (node.lineNr == pLineNr )
            {
                return node;
            }
        }
        
        return null;
    }

    /**
     * @return
     */
    public Iterator getMatrixConnectionIterator()
    {
        final List connVoList = getMatrixConnetionList();
        return connVoList.iterator();
    }

    /**
     * @return
     */
    public Iterator getMatrixConnectionIteratorSortedByDistance()
    {
        final List connVoList = getMatrixConnetionList();
        // sort by distance, siehe tricky comparator
        Collections.sort( connVoList );
//System.out.println( "connVoList: [" + connVoList.size() + "]" + connVoList );        
        return connVoList.iterator();
    }

    /**
     * @return
     */
    public Iterator getMatrixConnectionIteratorSortedByDistanceAndJoinedForBidirectionalLinks()
    {
        final List connVoList = getMatrixConnetionList();
        
        final List joinedConnVoList = 
            joinForBidirectionalLinks(
                    connVoList );
        
        // sort by distance, siehe tricky comparator
        Collections.sort( joinedConnVoList );
//System.out.println( "joinedConnVoList: [" + connVoList.size() + "]" + joinedConnVoList );        
        return joinedConnVoList.iterator();
    }

    /**
     * 
     * @param pConnVoList
     * @return
     */
    private static List joinForBidirectionalLinks(
            List pConnVoList)
    {
        final int connVoListSize = pConnVoList.size();
        
        // TODO wieder auf HashSet umstellen, war nur HashMap fr develop, weil es kein Set.get(key) gibt fr die Kontrolle auf unterschiedliche Styles
        final HashMap voSet = new HashMap();
        
        for (Iterator iter = pConnVoList.iterator(); iter.hasNext();)
        {
            final AssignmentMatrixConnectionVO connVo = (AssignmentMatrixConnectionVO) iter.next();

            final AssignmentMatrixConnectionVO reverseConnVo =
                new AssignmentMatrixConnectionVO(
                        connVo.referencedNode,
                        connVo.node,
                        connVo.style,
                        false );
            
            if ( voSet.containsKey( reverseConnVo ) )
            {
                voSet.remove( reverseConnVo );

                final AssignmentMatrixConnectionVO bidiConnVo =
                    new AssignmentMatrixConnectionVO(
                            connVo.referencedNode,
                            connVo.node,
                            connVo.style,
                            true );
                voSet.put( bidiConnVo , bidiConnVo );
            }
            else
            {
                voSet.put( connVo, connVo );
            }
        }
        return Arrays.asList( voSet.values().toArray() );
    }

    /**
     * Zhlen der Verbindungen fr Develop-Check
     * 
     * @param pVoSet
     * @return
     */
    private static int countOfConnections(
            final HashMap pVoSet)
    {
        int connCount = 0;
        for (Iterator iter = pVoSet.values().iterator(); iter.hasNext();)
        {
            final AssignmentMatrixConnectionVO connVO = (AssignmentMatrixConnectionVO) iter.next();
            connCount += connVO.isBidirektional ? 2 : 1 ;
        }
        return connCount;
    }

    /**
     * @return
     */
    public List getMatrixConnetionList()
    {
//        final List connVoList = new ArrayList( this.getNodeCount() * 2 );
//        for (final Iterator nodeIter = this.nodeIterator(); nodeIter.hasNext();)
//        {
//            final AssignmentMatrixNode node = (AssignmentMatrixNode) nodeIter.next();
//            for (Iterator referencedNodeIter = node.referencedNodeIterator(); referencedNodeIter.hasNext();)
//            {
//                final AssignmentMatrixNode referencedNode = (AssignmentMatrixNode) referencedNodeIter.next();
//                
//                final AssignmentMatrixConnectionVO connVO =
//                    new AssignmentMatrixConnectionVO(
//                            node,
//                            referencedNode,
//                            false );
//                
//                connVoList.add( connVO );
//            }
//        }
//        return connVoList;
        
//System.out.println( "connVoSet: [" + connVoSet.size() + "]" + connVoSet );        
        return Arrays.asList( this.connVoSet.toArray() );
    }

    /**
     * 
     * @param pNodeName
     * @return
     */
    public AssignmentMatrixNode getNodeForName(
            final String pNodeName)
    {
        return (AssignmentMatrixNode) this.nameToNodeMap.get( pNodeName );
    }

    /**
     * 
     * @param pConnectionVO
     */
    public void addConnection(
            final AssignmentMatrixConnectionVO pConnectionVO)
    {
        boolean isNew = this.connVoSet.add( pConnectionVO );
        if ( ! isNew )
        {
            throw new RuntimeException( "connection already exists in matrix: " + pConnectionVO );
        }
    }

}
