package assignment_matrix_layouter;

/**
 * Stil-Klasse fr die Verbindungs-Linie
 * zweier Knoten.
 * 
 * Festgelegt wird
 * -Zeichen horizontal vom Knoten zur vertikalen Linie
 * -Zeichen Ecke (bergang horizontal zu vertikal) oben
 * -Zeichen vertikal
 * -Zeichen Ecke (bergang horizontal zu vertikal) unten
 * -Zeichen horizontal von der vertikalen Linie zum Knoten
 * 
 * Der Pfeil am Ende (grsser als und kleiner als Zeichen)
 * ist nicht beeinflussbar.
 * 
 * @author Heiner Kcker
 */
public final class AssignmentMatrixConnectionStyle
implements Comparable
{
    // --- static ----
    
    public static final AssignmentMatrixConnectionStyle DEFAULT_STYLE =
        new AssignmentMatrixConnectionStyle( 
                //"-+|+-" 
                //"-,|'-" 
                "-,|`-" ,
                "=*"
                );

    // --- non static ----
    
    /**
     * Zeichen-Stil zum Verbinden zweier 
     * unterschiedlicher Knoten.
     */
    public final String diffNodeStyleStr;

    /**
     * Zeichen-Stil zum Verbinden eines
     * Knotens mit sich selbst.
     */
    public final String equalNodeStyleStr;

    /**
     * Constructor.
     *
     * @param pDiffStyleStr
     */
    public AssignmentMatrixConnectionStyle(
            final String pDiffStyleStr,
            final String pEqualNodeStyleStr )
    {
        if ( pDiffStyleStr.length() != 5 )
        {
            throw new IllegalArgumentException( "length of diff style string must be 5 characters" );
        }
        this.diffNodeStyleStr = pDiffStyleStr;
        
        if ( pEqualNodeStyleStr.length() != 2 )
        {
            throw new IllegalArgumentException( "length of equal style string must be 2 characters" );
        }
        this.equalNodeStyleStr = pEqualNodeStyleStr;
    }
    
    /**
     * 
     * @return
     */
    public char getFromNodeLineChr()
    {
        return this.diffNodeStyleStr.charAt( 0 );
    }

    /**
     * 
     * @return
     */
    public char getTopCornerChr()
    {
        return this.diffNodeStyleStr.charAt( 1 );
    }
    
    /**
     * 
     * @return
     */
    public char getVerticalChr()
    {
        return this.diffNodeStyleStr.charAt( 2 );
    }
    
    /**
     * 
     * @return
     */
    public char getBottomCornerChr()
    {
        return this.diffNodeStyleStr.charAt( 3 );
    }
    
    /**
     * 
     * @return
     */
    public char getToNodeLineChr()
    {
        return this.diffNodeStyleStr.charAt( 4 );
    }

    /**
     * 
     * @return
     */
    public char getEqualFromToNodeLineChr()
    {
        return this.equalNodeStyleStr.charAt( 0 );
    }

    /**
     * 
     * @return
     */
    public char getEqualVerticalChr()
    {
        return this.equalNodeStyleStr.charAt( 1 );
    }

    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(
            final Object pObj )
    {
        final AssignmentMatrixConnectionStyle other = ( (AssignmentMatrixConnectionStyle) pObj );
        int retVal = this.diffNodeStyleStr.compareTo( other.diffNodeStyleStr );
        if ( retVal != 0 )
        {
            return retVal;
        }
        return this.equalNodeStyleStr.compareTo( other.equalNodeStyleStr );
    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    public int hashCode()
    {
        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + ((this.diffNodeStyleStr == null) ? 0 : this.diffNodeStyleStr.hashCode());
        result = PRIME * result + ((this.equalNodeStyleStr == null) ? 0 : this.equalNodeStyleStr.hashCode());
        return result;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj)
    {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final AssignmentMatrixConnectionStyle other = (AssignmentMatrixConnectionStyle) obj;
        if (this.diffNodeStyleStr == null)
        {
            if (other.diffNodeStyleStr != null)
                return false;
        }
        else if (!this.diffNodeStyleStr.equals(other.diffNodeStyleStr))
            return false;
        if (this.equalNodeStyleStr == null)
        {
            if (other.equalNodeStyleStr != null)
                return false;
        }
        else if (!this.equalNodeStyleStr.equals(other.equalNodeStyleStr))
            return false;
        return true;
    }

}
