package assignment_matrix_layouter;

/**
 * @author Heiner Kcker
 */
public class AssignmentMatrixConnectionVO
implements Comparable
{
    /**
     * Der eigentliche Knoten 
     * (von hier geht der Pfeil aus)
     */
    final AssignmentMatrixNode node;
    
    /**
     * Der verbundene Knoten 
     * (auf den der eigentliche zeigt)
     * (hier geht der Pfeil hin)
     */
    final AssignmentMatrixNode referencedNode;
    
    /**
     * Die Art (Style) der Verbindungs-Linie
     */
    final AssignmentMatrixConnectionStyle style;
    
    /**
     * ob diese Verbindung in beide Richtungen geht.
     */
    final boolean isBidirektional;
    
    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    public int hashCode()
    {
        final int PRIME = 31;
        int result = 1;
        result = PRIME * result + (this.isBidirektional ? 1231 : 1237);
        result = PRIME * result + ((this.node == null) ? 0 : this.node.hashCode());
        result = PRIME * result + ((this.referencedNode == null) ? 0 : this.referencedNode.hashCode());
        result = PRIME * result + ((this.style == null) ? 0 : this.style.hashCode());
        return result;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj)
    {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final AssignmentMatrixConnectionVO other = (AssignmentMatrixConnectionVO) obj;
        if (this.isBidirektional != other.isBidirektional)
            return false;
        if (this.node == null)
        {
            if (other.node != null)
                return false;
        }
        else if (!this.node.equals(other.node))
            return false;
        if (this.referencedNode == null)
        {
            if (other.referencedNode != null)
                return false;
        }
        else if (!this.referencedNode.equals(other.referencedNode))
            return false;
        if (this.style == null)
        {
            if (other.style != null)
                return false;
        }
        else if (!this.style.equals(other.style))
            return false;
        return true;
    }

    /**
     * Constructor.
     *
     * @param pNode
     * @param pReferencedNode
     */
    public AssignmentMatrixConnectionVO(
            final AssignmentMatrixNode pNode, 
            final AssignmentMatrixNode pReferencedNode,
            final AssignmentMatrixConnectionStyle pStyle,
            final boolean pIsBidirektional)
    {
        // TODO avoid null
        this.node = pNode;
        this.referencedNode = pReferencedNode;
        this.style = pStyle;
        this.isBidirektional = pIsBidirektional;
    }
    
    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(
            final Object pO)
    {
        final AssignmentMatrixConnectionVO otherConnectionVO = ( ( AssignmentMatrixConnectionVO) pO );
        // TODO ich bescheisse hier ein bisschen, weil ich nach der Distanz sortiere, um zu erreichen, dass die krzesten strecken zuerst gezeichnet werden
        int distanceDiff = this.getNodeDistance() - otherConnectionVO.getNodeDistance();
        if ( distanceDiff != 0 )
        {
            return distanceDiff;
        }
        // Untervergleichs-Kriterium Position
        distanceDiff = this.getTopLine() - otherConnectionVO.getTopLine();
        if ( distanceDiff != 0 )
        {
            return distanceDiff;
        }
        // Untervergleichs-Kriterium node1
        distanceDiff = this.node.compareTo( otherConnectionVO.node );
        if ( distanceDiff != 0 )
        {
            return distanceDiff;
        }
        // Untervergleichs-Kriterium node2
        distanceDiff = this.referencedNode.compareTo( otherConnectionVO.referencedNode );
        if ( distanceDiff != 0 )
        {
            return distanceDiff;
        }
        // Vergleich nach Style
        return this.style.compareTo( otherConnectionVO.style );
    }
    
    /**
     * Get oberste Zeile der verbundenen Knoten.
     * 
     * @return
     */
    public int getTopLine()
    {
        return Math.min( 
                this.node.lineNr , 
                this.referencedNode.lineNr );
    }
    
    /**
     * Distance der beiden Knoten absolut (nicht vorzeichenbehaftet)
     * @return
     */
    private int getNodeDistance()
    {
        return Math.abs( 
                this.node.lineNr 
                - this.referencedNode.lineNr );
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return "node: " + this.node +" "
            + "referencedNode: " + this.referencedNode + " "
            + "isBidirektional: " + this.isBidirektional;
    }

}
