package assignment_matrix_layouter;

import util.StringUtil;

/**
 * @author Heiner Kcker
 */
public final class AssignmentMatrixNode
implements Comparable
{
    /**
     * Name dieses Knotens
     */
    final String name;

    /**
     * Die Zeilen-Nummer, in der dieser Knoten angezeigt wird.
     */
    final int lineNr;

    /**
     * Menge der {@link AssignmentMatrixNode}-Objekte,
     * auf welche diese {@link AssignmentMatrixNode} eine
     * Referenz hat.
     */
    //private final Set hasReferenceToNodeSet = new TreeSet();
    
    /**
     * Referenz auf die Zuordnungs-Matrx, in welcher
     * diese {@link AssignmentMatrixNode} enthalten ist.
     */
    AssignmentMatrix matrix;

    /**
     * Constructor.
     *
     * @param pName
     * @param pLine
     */
    public AssignmentMatrixNode(
            final String pName, 
            final int pLine)
    {
        // TODO avoid null pointer, avoid ambigous (same line) node
        this.name = pName;
        // TODO ensure line >= 0
        this.lineNr = pLine;
    }
    
    /**
     * add.
     * 
     * @param pOtherNode
     */
    public void addReferenceTo(
            final AssignmentMatrixNode pOtherNode)
    {
        addReferenceTo(
                pOtherNode,
                AssignmentMatrixConnectionStyle.DEFAULT_STYLE );
    }
    /**
     * add.
     * 
     * @param pOtherNode
     */
    public void addReferenceTo(
            final AssignmentMatrixNode pOtherNode, 
            final AssignmentMatrixConnectionStyle pStyle )
    {
        if ( pOtherNode.equals( this ) )
        {
            // TODO optional eine Verbindung auf sich selbst erlauben, aber wie soll ich das zeichnen (noch klren)
            //throw new IllegalArgumentException( "reference to this (same node) not permitted: " + this );
        }

        /*
         * Neue Vorgehensweise mit Verwalten aller Verbindungen in der ..Matrix-Klasse
         */
        if ( this.matrix == null )
        {
            throw new RuntimeException( "add this node before to matrix and thereafter node to node: " + this );
        }
        
        
        this.matrix.addConnection(
                new AssignmentMatrixConnectionVO( 
                        this,
                        pOtherNode,
                        pStyle,
                        false ) );
        
    }
    
    /**
     * 
     * @param pOtherNode
     * @return
     */
//    public boolean hasReferenceTo(
//            final AssignmentMatrixNode pOtherNode)
//    {
//        return this.hasReferenceToNodeSet.contains( pOtherNode );
//    }

    /* (non-Javadoc)
     * @see java.lang.Object#hashCode()
     */
    public int hashCode()
    {
        final int PRIME = 31;
        int result = 1;
        // nur Vergleich Zeile
        result = PRIME * result + this.lineNr;
        return result;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj)
    {
        if (this == obj)
            return true;
        if (obj == null)
            return false;
        if (getClass() != obj.getClass())
            return false;
        final AssignmentMatrixNode other = (AssignmentMatrixNode) obj;
        // nur Vergleich Zeile
        if (this.lineNr != other.lineNr)
            return false;
        return true;
    }

    /* (non-Javadoc)
     * @see java.lang.Comparable#compareTo(java.lang.Object)
     */
    public int compareTo(
            final Object pOther)
    {
        // nur Vergleich Zeile
        return this.lineNr - ( (AssignmentMatrixNode) pOther ).lineNr;
    }

    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return StringUtil.getShortClassName( this ) + " lineNr: " + this.lineNr + " name: " + this.name;
    }

    /**
     * @return
     */
//    public Iterator referencedNodeIterator()
//    {
//        return Collections.unmodifiableSet( this.hasReferenceToNodeSet ).iterator();
//    }

}
