package assignment_matrix_layouter;

import java.util.*;

import util.test.BigBinary;

/**
 * Ich hoffe, dass ich hiermit alle Testflle des
 * [@link AssignmentMatrixLayouter} abdecken kann
 * und einen generisch verwendbaren Test-Generator
 * hieraus bauen kann.
 * 
 * @author Heiner Kcker
 */
public class AssignmentTestGenerator
{

    /**
     * @param args
     */
    public static void main(String[] args)
    {
        for (int i = 1; i < 9; i++)
        {
            generateTestForNodeCount( i + 1 );
        }
    }

    /**
     * Alle Permutationen von Verbindungen werden erzeugt.
     * 
     * @param pNodeCount
     */
    private static void generateTestForNodeCount(
            final int pNodeCount)
    {
        /*
         * Alle Permutationen von verbindungen werden erzeugt.
         */
       final ArrayList connList = new ArrayList( pNodeCount * ( pNodeCount - 1 ) ) ;
        for (int i = 0; i < pNodeCount; i++)
        {
            for (int j = 0; j < pNodeCount; j++)
            {
                //if ( i != j )
                    // keine verbindung auf sich selbst erlaubt
                {
                    connList.add(
                            new NodeNrConnection(
                                    i,
                                    j ) );
                }
            }
        }
        
        final NodeNrConnection[] nodeNrConnArr = 
            (NodeNrConnection[]) 
            connList.toArray( new NodeNrConnection[ connList.size() ] );
        
        generateTestPermutationsForNodeConnectionArr(
                nodeNrConnArr, 
                pNodeCount );
    }
    
    /**
     * Die bergebenen Verbindungen werden in allen mglichen
     * Kombinationen getestet.
     * 
     * Zum erzeugen der Kombinationen dient ein {@link BigBinary}.
     * 
     * @param pConnList
     */
    private static void generateTestPermutationsForNodeConnectionArr(
            final NodeNrConnection[] pNodeNrConnectionArr , 
            final int pNodeCount )
    {
        final BigBinary bigBinary = new BigBinary( pNodeNrConnectionArr.length );
        
        while ( ! bigBinary.carry() )
        {
            final NodeNrConnection[] runNodeNrConnArr =
                (NodeNrConnection[])
                bigBinary.filter( pNodeNrConnectionArr );
            
            if ( runNodeNrConnArr.length > 0 )
            {
                generateTestForNodeConnectionArr(
                        runNodeNrConnArr,
                        pNodeCount );
            }
            
            bigBinary.increment();
        }
    }

    /**
     * Die bergebenen Verbindungen werden auslayoutet.
     * 
     * @param pRunNodeNrConnArr
     */
    private static void generateTestForNodeConnectionArr(
            final NodeNrConnection[] pNodeNrConnArr, 
            final int pNodeCount )
    {
        /*
         * Anlegen der Knoten
         */
        AssignmentMatrix matrix = new AssignmentMatrix();
        
        final AssignmentMatrixNode[] nodeArr = new AssignmentMatrixNode[ pNodeCount ];
        
        for (int i = 0; i < pNodeCount; i++)
        {
            final AssignmentMatrixNode node = 
                new AssignmentMatrixNode( 
                        "Node" + i,
                        1 + ( i * 2 ) );
            nodeArr[ i ] = node;
            matrix.add( node );
        }

        /*
         * Verknpfen der Knoten
         */
        for (int i = 0; i < pNodeNrConnArr.length; i++)
        {
            final NodeNrConnection connection = pNodeNrConnArr[i];
            
            nodeArr[ connection.nodeNr1 ].addReferenceTo( 
                    nodeArr[ connection.nodeNr2 ] );
        }

        System.out.println( Arrays.asList( pNodeNrConnArr ) );
        System.out.println( "JoinBidirectionalLinks: false" );

        System.out.println(
                AssignmentMatrixLayouter.createLayout(
                        matrix , 
                        " " , // pRightBorderStr, 
                        false ) );
        
        System.out.println( "===================================================" );
        
        System.out.println( Arrays.asList( pNodeNrConnArr ) );
        System.out.println( "JoinBidirectionalLinks: true" );

        System.out.println(
                AssignmentMatrixLayouter.createLayout(
                        matrix , 
                        " " , // pRightBorderStr, 
                        true ) );

        System.out.println( "===================================================" );

    }

    /**
     * Hilfsklasse (Value-Object) 
     * mit jeweils zwei verbundenen 
     * Knoten-Nummern
     */
    static class NodeNrConnection
    {
        final int nodeNr1;
        final int nodeNr2;
        /**
         * Constructor.
         *
         * @param pNodeNr1
         * @param pNodeNr2
         */
        public NodeNrConnection(
                final int pNodeNr1, 
                final int pNodeNr2)
        {
            this.nodeNr1 = pNodeNr1;
            this.nodeNr2 = pNodeNr2;
        }
        /* (non-Javadoc)
         * @see java.lang.Object#toString()
         */
        public String toString()
        {
            return this.nodeNr1 + "->" + this.nodeNr2;
        }
    }

}
