package util;

import java.util.*;

/**
 * Hilfs-Methoden zum formatierten Ausgeben von List's und Map's.
 * 
 * @author Heiner Kcker
 */
public class CollectionFormatUtil
{

    /**
     * Constructor private, dont instantiatate this class.
     */
    private CollectionFormatUtil()
    {
        super();
    }

    /**
     * Schalter fr streng (Exception wenn Wert nicht gefunden) oder nicht
     * (return null)
     */
    public static final boolean STRICT              = false;

    /** Einrckung fr die formatierte Auagabe von List' oder Map's */
    public static final String  INDENT              = "  ";

    /** Zeilenumbruch fr die formatierte Auagabe von List' oder Map's */
    public static final char    NL                  = '\n';

    /** Schlssel Wert Trenner fr die formatierte Auagabe von List' oder Map's */
    public static final String  KEY_VALUE_SEPERATOR = " => ";

    /**
     * Formatierte Ausgabe des bergebenen Arrays.
     * 
     * @param pArr
     * @return
     */
    public static final String toStr( // --
            final Object[] pArr)
    {
        if (pArr == null)
        {
            return null;
        }
        return toStr(Arrays.asList(pArr));
    }

    /**
     * Formatierte Ausgabe der bergebenen List.
     * 
     * @param pList
     * @return
     */
    public static final String toStr( // --
            final List pList)
    {
        if (pList == null)
        {
            return null;
        }
        return toStr(pList, "");
    }

    /**
     * Formatierte Ausgabe der bergebenen Map.
     * 
     * @param pMap
     * @return
     */
    public static final String toStr( // --
            final Map pMap)
    {
        if (pMap == null)
        {
            return null;
        }
        return toStr(pMap, "");
    }

    /**
     * Rekursive Methode zur formatierten Ausgabe der bergebenen List.
     * 
     * @param pList
     * @param pIndent
     * @return
     */
    private static String toStr(// --
            final List pList, // --
            final String pIndent)
    {
        int i = 0;
        final UnSynchStringBuffer strBuff = new UnSynchStringBuffer(1024);
        // for (Object obj : pList)
        for (Iterator iter = pList.iterator(); iter.hasNext();)
        {
            final Object value = iter.next();

            strBuff.append(NL);
            strBuff.append(pIndent);
            strBuff.append("[");
            strBuff.append(String.valueOf(i++));
            strBuff.append("]");

            toStr(pIndent, strBuff, value);
        }
        return strBuff.toString();
    }

    /**
     * Rekursive Methode zur formatierten Ausgabe der bergebenen Map.
     * 
     * @param pMap
     * @param pIndent
     * @return
     */
    private static String toStr(// --
            final Map pMap, // --
            final String pIndent)
    {
        final UnSynchStringBuffer strBuff = new UnSynchStringBuffer(1024);
        final Set/* <Map.Entry> */entrySet = pMap.entrySet();
        // for (Map.Entry entry : entrySet)
        for (Iterator iter = entrySet.iterator(); iter.hasNext();)
        {
            Map.Entry entry = (Map.Entry) iter.next();
            strBuff.append(NL);
            strBuff.append(pIndent);
            strBuff.append("(");
            strBuff.append(entry.getKey());
            strBuff.append(")");

            final Object value = entry.getValue();

            toStr(pIndent, strBuff, value);
        }
        return strBuff.toString();
    }

    /**
     * @param pIndent
     * @param pStrBuff
     * @param pValue
     */
    private static void toStr(
            final String pIndent, 
            final UnSynchStringBuffer pStrBuff, 
            final Object pValue)
    {
        if (pValue instanceof Object[])
        {
            pStrBuff.append(NL);
            pStrBuff.append(pIndent);
            pStrBuff.append("[");
            pStrBuff.append(toStr(Arrays.asList((Object[]) pValue), pIndent
                    + INDENT));
            pStrBuff.append(NL);
            pStrBuff.append(pIndent);
            pStrBuff.append("]");
        }
        else if (pValue instanceof List)
        {
            pStrBuff.append(NL);
            pStrBuff.append(pIndent);
            pStrBuff.append("[");
            pStrBuff.append(toStr((List) pValue, pIndent + INDENT));
            pStrBuff.append(NL);
            pStrBuff.append(pIndent);
            pStrBuff.append("]");
        }
        else if (pValue instanceof Map)
        {
            pStrBuff.append(NL);
            pStrBuff.append(pIndent);
            pStrBuff.append("{");
            pStrBuff.append(toStr((Map) pValue, pIndent + INDENT));
            pStrBuff.append(NL);
            pStrBuff.append(pIndent);
            pStrBuff.append("}");
        }
        else
        {
            pStrBuff.append(KEY_VALUE_SEPERATOR);
            if (pValue instanceof CharSequence)
            {
                // String Einschliessen in Hochkommas, damit Leerstrings und
                // am auftauchende Leerzeichen erkannt werden knnen
                pStrBuff.append("'" + pValue + "'");
            }
            else
            {
                pStrBuff.append(pValue);
            }
        }
    }

}
