package util;

import java.util.*;

/**
 * Allgemeine Hilfs-Methoden fr
 * Collections.
 * 
 * @author Heiner Kcker
 */
public final class CollectionUtil
{
    /**
     * Constructor.
     */
    private CollectionUtil()
    {
        super();
    }

    /**
     * Diese Methode liefert die Schnittmenge
     * (intersection) zweier bergebener Mengen.
     * Die Ergebnis-Menge enthlt alle Elemente,
     * die in beiden bergebenen Mengen
     * enthalten sind.
     * 
     * @param pSet1
     * @param pColl2
     * @return
     */
    public static HashSet intersection(
            final HashSet pSet1,
            final Collection pColl2 )
    {
        // shallow copy (flache Kopie)
        final HashSet retSet = (HashSet) pSet1.clone();
        
        retSet.retainAll( pColl2 );
        
        return retSet;
    }

    /**
     * 
     * @param pSet
     * @param pCollToRemove
     * @return
     */
    public static HashSet removeAll(
            final HashSet pSet, 
            final Collection pCollToRemove )
    {
        // shallow copy (flache Kopie)
        final HashSet retSet = (HashSet) pSet.clone();
        
        retSet.removeAll( pCollToRemove );
        
        return retSet;
    }

    /**
     * Einlesen einer zeilenweise
     * getrennten Datei mit Long-Zahlen 
     * in ein HashSet.
     * 
     * @param pInFileNameWithPath
     * @return
     */
    public static HashSet file2LongHashSet(
            String pInFileNameWithPath )
    {
        final String[] lineStrArr = StringUtil.fileToStrArr( pInFileNameWithPath );
        final HashSet retSet = new HashSet( lineStrArr.length );
        for (int i = 0; i < lineStrArr.length; i++)
        {
            final String lineStr = lineStrArr[i];
            final Long longObj = StringUtil.str2LongLenient( lineStr );
            if ( longObj == null )
            {
                System.out.println( "nicht lesbare Zeile: '" + lineStr + "'" );
            }
            else
                // gltiges long
            {
                retSet.add( longObj );
            }
        }
        return retSet;
    }

    /**
     * Sortiertes Einlesen einer zeilenweise
     * getrennten Datei mit Long-Zahlen 
     * in eine ArrayList.
     * 
     * @param pInFileNameWithPath
     * @return
     */
    public static ArrayList file2LongListSorted(
            String pInFileNameWithPath )
    {
        final ArrayList list =
            file2LongList(
                    pInFileNameWithPath);
        Collections.sort( list );
        return list;
    }
    /**
     * Einlesen einer zeilenweise
     * getrennten Datei mit Long-Zahlen 
     * in eine ArrayList.
     * 
     * @param pInFileNameWithPath
     * @return
     */
    public static ArrayList file2LongList(
            String pInFileNameWithPath )
    {
        System.out.println( "Lese: " + pInFileNameWithPath );
        final String[] lineStrArr = StringUtil.fileToStrArr( pInFileNameWithPath );
        final ArrayList retList = new ArrayList( lineStrArr.length );
        for (int i = 0; i < lineStrArr.length; i++)
        {
            final String lineStr = lineStrArr[i];
            final Long longObj = StringUtil.str2LongLenient( lineStr );
            if ( longObj == null )
            {
                System.out.println( "nicht lesbare Zeile: '" + lineStr + "'" );
            }
            else
                // gltiges long
            {
                retList.add( longObj );
            }
        }
        return retList;
    }

    /**
     * Einlesen einer zeilenweise
     * getrennten Datei in ein
     * HashSet mit Strings.
     * 
     * @param pInFileNameWithPath
     * @return
     */
    public static HashSet file2StrHashSet(
            String pInFileNameWithPath )
    {
        final String[] lineStrArr = StringUtil.fileToStrArr( pInFileNameWithPath );
        final HashSet retSet = new HashSet( lineStrArr.length );
        for (int i = 0; i < lineStrArr.length; i++)
        {
            final String lineStr = lineStrArr[i];
            // TODO optional trim
            retSet.add( lineStr );
        }
        return retSet;
    }

    /**
     * write set to file separated with cr and lf.
     * 
     * @param pSet
     * @param pOutFileNameWithPath
     */
//    public static void set2file(
//            final Set pSet, 
//            final String pOutFileNameWithPath )
//    {
//        set2file(
//                pSet, 
//                pOutFileNameWithPath ,
//                "\r\n" );
//    }

    /**
     * write set to file with specified separator string.
     * 
     * @param pSet
     * @param pOutFileNameWithPath
     * @param pString
     */
//    private static void set2file(
//            final Set pSet, 
//            final String pOutFileNameWithPath, 
//            final String pSeparationStr )
//    {
//        final UnSynchStringBuffer outStrBuff = new UnSynchStringBuffer();
//
//        synchronized ( outStrBuff )
//            // synchronized fr Beschleunigung siehe Javainsel
//        {
//            for (Iterator iter = pSet.iterator(); iter.hasNext();)
//            {
//                final Object elemObj = iter.next();
//                outStrBuff.append( elemObj + pSeparationStr );
//            }
//            StringUtil.str2file( outStrBuff.toString() , pOutFileNameWithPath );
//        }
//    }

    /**
     * write set to file separated with cr and lf.
     * 
     * @param pTnrOhneMahnungenSet
     * @param pOutFileNameWithPath
     */
    public static void collection2file(
            final Collection pCollection, 
            final String pOutFileNameWithPath )
    {
        collection2file(
                pCollection, 
                pOutFileNameWithPath ,
                "\r\n" );
    }

    /**
     * write set to file separated with cr and lf
     * with sorting.
     * 
     * @param pTnrOhneMahnungenSet
     * @param pOutFileNameWithPath
     */
    public static void collection2fileSorted(
            final Collection pCollection, 
            final String pOutFileNameWithPath )
    {
        collection2fileSorted(
                pCollection, 
                pOutFileNameWithPath ,
                "\r\n" );
    }

    /**
     * write Collection to file with specified separator string.
     * 
     * @param pSet
     * @param pOutFileNameWithPath
     * @param pString
     */
    public static void collection2file(
            final Collection pCollection, 
            final String pOutFileNameWithPath, 
            final String pSeparationStr )
    {
        final UnSynchStringBuffer outStrBuff = new UnSynchStringBuffer( pCollection.size() * 20 );

        //synchronized ( outStrBuff )
            // synchronized fr Beschleunigung siehe Javainsel
        {
            for (Iterator iter = pCollection.iterator(); iter.hasNext();)
            {
                final Object elemObj = iter.next();
                outStrBuff.append( elemObj + pSeparationStr );
            }
            StringUtil.str2file( outStrBuff.toString() , pOutFileNameWithPath );
        }
    }

    /**
     * write Collection to file with specified separator string
     * with sorting.
     * 
     * @param pSet
     * @param pOutFileNameWithPath
     * @param pString
     */
    public static void collection2fileSorted(
            final Collection pCollection, 
            final String pOutFileNameWithPath, 
            final String pSeparationStr )
    {
        final ArrayList sortList = new ArrayList( pCollection );
        
        CollectionUtil.sortCollection( sortList );
        
        final UnSynchStringBuffer outStrBuff = new UnSynchStringBuffer( pCollection.size() * 20 );

        //synchronized ( outStrBuff )
            // synchronized fr Beschleunigung siehe Javainsel
        {
            for (Iterator iter = sortList.iterator(); iter.hasNext();)
            {
                final Object elemObj = iter.next();
                outStrBuff.append( elemObj + pSeparationStr );
            }
            StringUtil.str2file( outStrBuff.toString() , pOutFileNameWithPath );
        }
    }

    /**
     * write set to System.out separated with cr and lf.
     * 
     * @param pTnrOhneMahnungenSet
     */
    public static void collection2sysout(
            final Collection pCollection )
    {
        collection2sysout(
                pCollection, 
                "\r\n" );
    }

    /**
     * write Collection to System.out with specified separator string.
     * 
     * @param pSet
     * @param pOutFileNameWithPath
     * @param pString
     */
    private static void collection2sysout(
            final Collection pCollection, 
            final String pSeparationStr )
    {
        final UnSynchStringBuffer outStrBuff = new UnSynchStringBuffer( pCollection.size() * 20 );

        //synchronized ( outStrBuff )
            // synchronized fr Beschleunigung siehe Javainsel
        {
            for (Iterator iter = pCollection.iterator(); iter.hasNext();)
            {
                final Object elemObj = iter.next();
                outStrBuff.append( elemObj + pSeparationStr );
            }
            System.out.println( outStrBuff.toString() );
        }
    }


    /**
     * Remove all Key-Values 
     * from pMap which contains in
     * pKeySetToRemove.
     * 
     * @param pMap Base Map
     * @param pKeySetToRemove Set with keys to remove from pMap
     * @return removed Map
     */
    public static HashMap removeAllKeys(
            final HashMap pMap, 
            final Set pKeySetToRemove )
    {
        // shallow copy (flache Kopie)
        final HashMap retMap = (HashMap) pMap.clone();
        
        for (Iterator iter = pKeySetToRemove.iterator(); iter.hasNext();)
        {
            final Object keyToRemove = iter.next();
            
            retMap.remove( keyToRemove );
        }
        
        return retMap;
    }

    /**
     * Retain (behalte) all Key-Values 
     * from pMap which contains in
     * pKeySetToRemove.
     * 
     * @param pMap Base Map
     * @param pKeySetToRemove Set with keys to remove from pMap
     * @return removed Map
     */
    public static HashMap retainAllKeys(
            final HashMap pMap, 
            final Set pKeySetToRetain )
    {
        final HashMap retMap = new HashMap();
        
        for (Iterator iter = pKeySetToRetain.iterator(); iter.hasNext();)
        {
            final Object keyToRetain = iter.next();
            
            if ( pMap.containsKey( keyToRetain ) )
                // Benutzung der Methode containsKey, weil der value auch null sein kann
            {
                Object valObj = pMap.get( keyToRetain );
                retMap.put(keyToRetain, valObj );
            }
        }
        return retMap;
    }

    /**
     * Einlesen einer zeilenweisen Text-Daten-Datei
     * in ein Set mit Closure fr Key und Value.
     * 
     * @param pInFileNameWithPath
     * @param pMapper Closure zum Liefern von Map-Key und Map-Value
     * @return
     */
    public static HashMap file2LinedHashMap(
            final String pInFileNameWithPath, 
            final LineStrToSetKeyValueMapper pMapper )
    {
        final String[] lineStrArr = StringUtil.fileToStrArr( pInFileNameWithPath );
        final HashMap retMap = new HashMap();
        for (int i = 0; i < lineStrArr.length; i++)
        {
            final String lineStr = lineStrArr[i];
            if ( pMapper.isOk( i , lineStr ) )
            {
                final Object mapKey = pMapper.getKey( i , lineStr );
                final Object mapValue = pMapper.getValue( i , lineStr );
                retMap.put( mapKey, mapValue );
            }
        }
        return retMap;
    }
    
    /**
     * Closure-Klasse zum Zuordnen des Key und des Values
     * fr die Ziel-Map aus einer Text-Daten-Datei-Zeile.
     */
    public static abstract class LineStrToSetKeyValueMapper
    {
        /**
         * Zurckgeben, ob die Zeile in die Ergebnis-Map bernommen werden soll ?
         * @param pLineNr Zeilen-Nummer Basis 0
         * @param pLineStr Zeilen-String
         * @return ob die Zeile in die Ergebnis-Map bernommen werden soll
         */
        public abstract boolean isOk(
                final int pLineNr,
                final String pLineStr);
        /**
         * Zurckgeben des Map-Key.
         * @param pLineNr Zeilen-Nummer Basis 0
         * @param pLineStr Zeilen-String
         * @return Key
         */
        public abstract Object getKey(
                final int pLineNr,
                final String pLineStr);
        /**
         * Zurckgeben des Map-Value.
         * @param pLineNr Zeilen-Nummer Basis 0
         * @param pLineStr Zeilen-String
         * @return Value
         */
        public abstract Object getValue(
                final int pLineNr,
                final String pLineStr);
    }

    /**
     * Zurckgeben sortierte Collection ohne
     * Seiteneffekt auf die bergebene
     * Collection.
     * 
     * @param pColl
     * @return
     */
    public static Collection sortCollection(
            final Collection pColl)
    {
        if ( pColl instanceof Set )
        {
            final TreeSet retSet = new TreeSet( pColl );
            return retSet;
        }
        if ( pColl instanceof List //--
                || pColl instanceof AbstractCollection )
        {
            final ArrayList retList = new ArrayList( pColl );
            Collections.sort( retList );
            return retList;
        }
        throw new IllegalArgumentException( "" + pColl.getClass() );
    }

    /**
     * Zurckgeben sortierte Map ohne
     * Seiteneffekt auf die bergebene
     * Map.
     * 
     * @param pColl
     * @return
     */
    public static Map sortMap(
            final Map pMap)
    {
        final TreeMap retMap = new TreeMap( pMap );
        return retMap;
    }

    /**
     * Ermittlung des ersten Key-Objektes der
     * bergebenen Map.
     *
     * @param pMap
     * @return das erste Key-Objekt
     */
    public static final Object getFirstKey(//--
        final Map pMap)
    {
      final Set keySet = pMap.keySet();
      for (Iterator iter = keySet.iterator(); iter.hasNext();)
      {
        return iter.next();
      }
      return null;
    }

    /**
     * Schalter fr die Betriebsart der {@link #path}-Methode 
     * streng (Exception wenn Wert nicht gefunden) oder 
     * nicht streng (return null)
     */
    public static final boolean STRICT = false;


    /**
     * Abfrage eines Wertes aus einer hierarchischen 
     * Collection aus Map's und List's mit einer 
     * beliebigen Anzahl als Pfad verknpfter Key-Objekte.
     *
     * @param pCollection
     * @param pKeyPathArr
     * @return
     */
    public static Object pathGet( //--
        final Object pCollection, //--
        final Object[] pKeyPathArr)
    {
      Object tVal = pCollection;
      for ( int i = 0 ; i < pKeyPathArr.length; i++ )
      {
        final Object keyRunObj = pKeyPathArr[ i ];
        if ( tVal == null )
        {
          if (STRICT)
          {
            // bei scharfer (intoleranter) Variante eine Exception werfen
            throw new RuntimeException( "no value found for param [" + keyRunObj + "] from path " + Arrays.asList( pKeyPathArr ) );
          }
          return null;
        }
        if ( tVal instanceof Map )
        {
          Map map = (Map) tVal;
          tVal = map.get( keyRunObj );
        }
        else if ( tVal instanceof List //--
            && keyRunObj instanceof Integer )
        {
          List list = (List) tVal;
          tVal = list.get( ( (Integer) keyRunObj ).intValue() );
        }
        else
        {
          if (STRICT)
          {
            // bei scharfer (intoleranter) Variante eine Exception werfen
            throw new RuntimeException( "no value found for param [" + keyRunObj + "] from path " + Arrays.asList( pKeyPathArr ) );
          }
          return null;
        }
      }
      return tVal;
    }

    /**
     * Abfrage einer hierarchischen Collection aus
     * Map's und List's mit einer beliebigen Anzahl
     * als Pfad verknpfter Key-Objekte.
     *
     * @param pCollection
     * @param pKeyPathArr
     * @return
     */
    public static Object pathGet( //--
        final Object pCollection, //--
        // mit Slash (Schrgstrich) separierter Pfad-String
        // Derzeit nur fr int-Parameter (Array- oder List-Index)
        final String pKeyPathStr)
    {
        final Object[] pathObjArr = convertPathStrToPathObjArr(pKeyPathStr);

        return pathGet(
                pCollection,
                pathObjArr );
    }

    /**
     * Setzen eines Wertes aus einer hierarchischen 
     * Collection aus Map's und List's mit einer 
     * beliebigen Anzahl als Pfad verknpfter Key-Objekte.
     *
     * @param pCollection
     * @param pKeyPathArr
     * @return old value
     */
    public static Object pathSet( //--
        final Object pCollection, //--
        final Object[] pKeyPathArr, //--
        final Object pValue)
    {
      Object tVal = pCollection;
      for ( int i = 0 ; i < pKeyPathArr.length; i++ )
      {
        final Object keyRunObj = pKeyPathArr[ i ];
        if ( tVal == null )
        {
            throw new RuntimeException( "path invalid for param [" + keyRunObj + "] from path " + Arrays.asList( pKeyPathArr ) );
        }
        if ( tVal instanceof Map )
        {
          final Map map = (Map) tVal;
          tVal = map.get( keyRunObj );
          if ( i >= pKeyPathArr.length - 1 )
              //Pfad-Ende erreicht
          {
              map.put( keyRunObj , pValue );
          }
        }
        else if ( tVal instanceof List //--
            && keyRunObj instanceof Integer )
        {
          final List list = (List) tVal;
          tVal = list.get( ( (Integer) keyRunObj ).intValue() );
          if ( i >= pKeyPathArr.length - 1 )
              //Pfad-Ende erreicht
          {
              list.set( ( (Integer) keyRunObj ).intValue() , pValue );
          }
        }
        else
        {
            throw new RuntimeException( "path invalid for param [" + keyRunObj + "] from path " + Arrays.asList( pKeyPathArr ) );
        }
      }
      return tVal;
    }

    /**
     * Setzen eines Wertes in einer hierarchischen Collection aus
     * Map's und List's mit einer beliebigen Anzahl
     * als Pfad verknpfter Key-Objekte.
     *
     * @param pCollection
     * @param pKeyPathArr
     * @return
     */
    public static Object pathSet( //--
        final Object pCollection, //--
        // mit Slash (Schrgstrich) separierter Pfad-String
        // Derzeit nur fr int-Parameter (Array- oder List-Index)
        final String pKeyPathStr,
        final Object pValue)
    {
        final Object[] pathObjArr = convertPathStrToPathObjArr(pKeyPathStr);

        return pathSet(
                pCollection,
                pathObjArr,
                pValue );
    }

    /**
     * Konvertieren eines mit Slash (Schrgstrich) separierten Pfad-String's.
     * Derzeit nur fr int-Parameter (Array- oder List-Index).
     * 
     * @param pKeyPathStr
     * @return
     */
    public static Object[] convertPathStrToPathObjArr(
            final String pKeyPathStr)
    {
        final String[] pathStrArr = 
            StringUtil.tokenize(
                    pKeyPathStr, 
                    "/", 
                    false, 
                    false );
        
        final Object[] pathObjArr = new Object[ pathStrArr.length ];

        for (int i = 0; i < pathStrArr.length; i++)
        {
            final String pathElemStr = pathStrArr[i];
            
            pathObjArr[ i ] = 
                StringUtil.str2IntegerLenient( 
                        pathElemStr );
        }
        return pathObjArr;
    }

    /**
     * Get Maximum.
     * 
     * @param 
     * @return
     */
    public static Comparable getMaximum(
            final Collection pColl)
    {
        if ( pColl == null )
        {
            return null;
        }
        Comparable maxValue = null;
        for (Iterator iter = pColl.iterator(); iter.hasNext();)
        {
            final Comparable elem = (Comparable) iter.next();
            if ( maxValue == null )
            {
                maxValue = elem;
            }
            else if ( elem != null && maxValue.compareTo( elem ) < 0 )
            {
                maxValue = elem;
            }
        }
        return maxValue;
    }

    /**
     * Get Minimum.
     * 
     * @param 
     * @return
     */
    public static Comparable getMinimum(
            final Collection pColl)
    {
        if ( pColl == null )
        {
            return null;
        }
        Comparable minValue = null;
        for (Iterator iter = pColl.iterator(); iter.hasNext();)
        {
            final Comparable elem = (Comparable) iter.next();
            if ( minValue == null )
            {
                minValue = elem;
            }
            else if ( elem != null && minValue.compareTo( elem ) > 0 )
            {
                minValue = elem;
            }
        }
        return minValue;
    }

    /**
     * Resize the given list if notwendig and 
     * set the given object at the given position.
     * 
     * @param pList
     * @param pPos
     * @param pValue
     */
    public static void set(
            final List pList, 
            final int pPos, 
            final Object pValue )
    {
        if ( pList instanceof ArrayList )
        {
            ( (ArrayList) pList ).ensureCapacity( pPos + 1 );
        }
        while ( pList.size() <= pPos )
            // resize list
        {
            pList.add( null );
        }
        pList.set( pPos, pValue );
    }

    /**
     * Vergrssere die bergeben Liste, wenn notwendig.
     * @param pList
     * @param pSize 
     */
    public static void resize(
            final List pList, 
            final int pSize )
    {
        if ( pList instanceof ArrayList )
        {
            ( (ArrayList) pList ).ensureCapacity( pSize );
        }
        while ( pList.size() < pSize )
            // resize list
        {
            pList.add( null );
        }
    }

    /**
     * Jedes Element darf nur einmal
     * vorkommen.
     * 
     * Achtung: {@link Object#hashCode()} 
     * und {@link Object#equals(Object)
     * mssen korrekt implementiert sein.
     *  
     * @param pColl
     * @return
     */
    public static ArrayList unifyToList(
            final Collection pColl )
    {
        if ( pColl == null )
        {
            return null;
        }
        final HashSet set = new HashSet( pColl );
        return new ArrayList(
                Arrays.asList( 
                        set.toArray() ) );
    }

    /**
     * Jedes Element darf nur einmal
     * vorkommen.
     * 
     * Achtung: {@link Object#hashCode()} 
     * und {@link Object#equals(Object)
     * mssen korrekt implementiert sein.
     *  
     * @param pColl
     * @return
     */
    public static HashSet unifyToSet(
            final Collection pColl )
    {
        if ( pColl == null )
        {
            return null;
        }
        return new HashSet( pColl );
    }

    /**
     * Umwandeln einer {@link java.util.List} mit 
     * String-Elementen in eine {@link java.util.List} mit 
     * Long-Elementen.
     * 
     * @param pList
     * @return
     */
    public static ArrayList strList2longList(
            final List pList)
    {
        final ArrayList retList = new ArrayList( pList.size() );
        for ( final Iterator iter = pList.iterator() ; iter.hasNext() ; )
        {
            final String lineStr = (String) iter.next();
            final Long longObj = StringUtil.str2LongLenient( lineStr );
            if ( longObj == null )
            {
                System.out.println( "nicht lesbare Zeile: '" + lineStr + "'" );
            }
            else
                // gltiges long
            {
                retList.add( longObj );
            }
        }
        return retList;
    }

    /**
     * @param args
     */
//    public static void main(String[] args)
//    {
//    }

}
