package util;

import java.io.*;

/**
 * Methoden zum Umgang mit Exceptions . <br/>
 *
 *
 * <br/>
 * <b>Author:</b> <a href="http://www.heinerkuecker.de" target="_blank">Heiner Kcker</a>
 * <br/>
 *
 * @author Heiner Kcker
 * @version $Id$
 */
public class ExceptionUtil 
{
    /**
     * Konstruktor private, instanziieren verboten
     */
    private ExceptionUtil() {;}
    
    /**
     * 
     * @deprecated benutze die Methode {@link Log#fatal(Exception)}
     * @param pThr
     */
    public static void printToErr(Throwable pThr)
    {
        System.err.println( pThr.getMessage() );
        pThr.printStackTrace();
    }
    
    /**
     * Zurckgeben des StackTrace einer 
     * Exception als String.
     *
     * @param tr zu bearbeitende Exception
     * @return Message und StackTrace als String
     */
    public static String getStackTrace(Throwable tr)
    {
      String  retString = "";

      if (tr != null)
      {
        StringWriter  writer = new StringWriter();
        PrintWriter   pwriter = new PrintWriter(writer);

        tr.printStackTrace(pwriter);

        retString = writer.toString();
      }

      return retString;
    }

   /**
    * Zurckgeben der Message und des StackTrace einer 
    * Exception als String.
    *
    * @param tr zu bearbeitende Exception
    * @return Message und StackTrace als String
    */
   public static String getMessageAndStackTrace(Throwable tr)
   {
     String  retString = "";

     if (tr != null)
     {
       retString = tr.getMessage();

       StringWriter  writer = new StringWriter();
       PrintWriter   pwriter = new PrintWriter(writer);

       tr.printStackTrace(pwriter);

       retString += "\n" + writer.toString();
     }

     return retString;
   } // end method getMessageAndStackTrace

   /**
    * Zurckgeben des StackTrace einer Exception 
    * als zeilenweises String-Array.
    *
    * @param pTr zu bearbeitende Exception
    * @return Message und StackTrace als zeilenweises String-Array
    */
   public static String[] getStackTraceArray(Throwable pTr)
   {
       return StringUtil.lineArray( getStackTrace( pTr ) );
   }

   /**
    * Zurckgeben der Message und des StackTrace einer Exception 
    * als zeilenweises String-Array.
    *
    * @param pTr zu bearbeitende Exception
    * @return Message und StackTrace als zeilenweises String-Array
    */
   public static String[] getMessageAndStackTraceArray(Throwable pTr)
   {
       return StringUtil.lineArray( getMessageAndStackTrace( pTr ) );
   }
}
