package util;

import java.util.*;

/**
 * Generated IntToIntHashMap class.
 * This class wraps an untyped HashMap
 * to an typed intTointHashMap
 * for type safe implementation.
 * 
 * TODO Betriebsartenschalter,ob bei nicht gefundenem Eintrag eine Exception geweorfen werden soll oder ein Default-Wert zurckgegeben wird.
 * TODO default-wert optional
 */
public class IntToIntHashMap
{
    /**
     * Diese Map wird typisiert.
     */
    final HashMap innerMap;

    /**
     * Constructs an empty <tt>intTointHashMap</tt> with the default initial capacity
     * (16) and the default load factor (0.75).
     */
    public IntToIntHashMap()
    {
        super();
        this.innerMap = new HashMap();
    }
    
    /**
     * Constructs an empty <tt>intTointHashMap</tt> with the specified initial
     * capacity and the default load factor (0.75).
     *
     * @param  initialCapacity the initial capacity.
     * @throws IllegalArgumentException if the initial capacity is negative.
     */
    public IntToIntHashMap(
            final int pInitialCapacity)
    {
        super();
        this.innerMap = new HashMap( pInitialCapacity );
    }

    /**
     * Constructs an empty <tt>HashMap</tt> with the specified initial
     * capacity and load factor.
     *
     * @param  initialCapacity The initial capacity.
     * @param  loadFactor      The load factor.
     * @throws IllegalArgumentException if the initial capacity is negative
     *         or the load factor is nonpositive.
     */
    public IntToIntHashMap(
            final int pInitialCapacity, 
            float pLoadFactor )
    {
        super();
        this.innerMap = new HashMap( pInitialCapacity, pLoadFactor );
    }
  
    /**
     * Constructs a new <tt>intTointHashMap</tt> with the same mappings as the
     * specified <tt>Map</tt>.  The <tt>intTointHashMap</tt> is created with
     * default load factor (0.75) and an initial capacity sufficient to
     * hold the mappings in the specified <tt>Map</tt>.
     *
     * @param   m the map whose mappings are to be placed in this map.
     * @throws  NullPointerException if the specified map is null.
     */
    public IntToIntHashMap(
            final Map pMap)
    {
        super();
        this.innerMap = new HashMap( pMap );
    }

    /**
     * 
     * @see java.util.Map#clear()
     */
    public void clear()
    {
        this.innerMap.clear();
    }

    /**
     * @return
     * @see java.lang.Object#clone()
     */
    public Object clone()
    {
        return new IntToIntHashMap( this.innerMap );
    }

    /**
     * @param pKey
     * @return
     * @see java.util.Map#containsKey(java.lang.Object)
     */
    public boolean containsKey(
            final int pKey)
    {
        return this.innerMap.containsKey( new Integer( pKey ) );
    }

    /**
     * @param pValue
     * @return
     * @see java.util.Map#containsValue(java.lang.Object)
     */
    public boolean containsValue(
            final int pValue)
    {
        return this.innerMap.containsValue(new Integer( pValue ) );
    }

    /**
     * @return
     * @see java.util.Map#entrySet()
     */
    public Set entrySet()
    {
        return this.innerMap.entrySet();
    }

    /**
     * @param pO
     * @return
     * @see java.util.AbstractMap#equals(java.lang.Object)
     */
    public boolean equals(
            final Object pObj)
    {
        if ( pObj instanceof IntToIntHashMap )
        {
            final IntToIntHashMap otherMap = (IntToIntHashMap) pObj;
            return this.innerMap.equals( otherMap.innerMap );
        }
        return false;
    }

    /**
     * @param pKey
     * @return
     * @see java.util.Map#get(java.lang.Object)
     */
    public int get(
            final int pKey)
    {
        return StringUtil.obj2int(
                this.innerMap.get( new Integer( pKey ) ),
                Integer.MIN_VALUE );
    }

    /**
     * @return
     * @see java.util.AbstractMap#hashCode()
     */
    public int hashCode()
    {
        return this.innerMap.hashCode();
    }

    /**
     * @return
     * @see java.util.Map#isEmpty()
     */
    public boolean isEmpty()
    {
        return this.innerMap.isEmpty();
    }

    /**
     * @return
     * @see java.util.Map#keySet()
     */
    public Set keySet()
    {
        return this.innerMap.keySet();
    }

    /**
     * @param pKey
     * @param pValue
     * @return
     * @see java.util.Map#put(java.lang.Object, java.lang.Object)
     */
    public int put(
            final int pKey, 
            final int pValue)
    {
        return StringUtil.obj2int(
                this.innerMap.put(
                        new Integer( pKey ), 
                        new Integer( pValue ) ),
                Integer.MIN_VALUE );
    }

    /**
     * @param pMap
     * @see java.util.Map#putAll(java.util.Map)
     */
    public void putAll(
            final IntToIntHashMap pMap)
    {
        this.innerMap.putAll( pMap.innerMap );
    }

    /**
     * @param pKey
     * @return
     * @see java.util.Map#remove(java.lang.Object)
     */
    public int remove(
            int pKey)
    {
        return StringUtil.obj2int(
                this.innerMap.remove( new Integer( pKey ) ),
                Integer.MIN_VALUE );
    }

    /**
     * @return
     * @see java.util.Map#size()
     */
    public int size()
    {
        return this.innerMap.size();
    }

    /**
     * @return
     * @see java.util.AbstractMap#toString()
     */
    public String toString()
    {
        return this.innerMap.toString();
    }

    /**
     * @return
     * @see java.util.Map#values()
     */
    public Collection values()
    {
        return this.innerMap.values();
    }
    
    /**
     * This method returns an clone
     * of the wrapped map object
     * to implement the {@link java.util.Map} interface.
     */
    public Map toMap()
    {
        return new HashMap( this.innerMap ) ;
    }
}
