package util;

import java.util.Arrays;

/**
 * Eigene unsynchronisierte StringBuffer-Implementierung.
 */
public class UnSynchStringBuffer
{
    /**
     * absolute Mindest-Vergrsserung
     */
    private static final int INCREASE_TRESHOLD = 32;
    
    /**
     * relative Mindest-Vergrsserung
     * TODO: bei Anwendung in float umstellen und
     * einen gebrochenen Wert angeben 
     * oder als int lassen und ein Vielfaches von 1
     * angeben.
     */
    private static final int INCREASE_FACTOR = 1;
    
    private int nextPos;
    private char[] chrArr;
    private final char[] nullArray = { 'n', 'u', 'l', 'l' };
    
    /**
     * 
     * Constructor.
     *
     * @param pLength
     */
    public UnSynchStringBuffer(
            final int pLength )
    {
        super();
        this.nextPos = 0;
        this.chrArr = new char[ pLength ];
    }
    /**
     * Constructor.
     *
     * @param pStr
     */
    public UnSynchStringBuffer(
            final String pStr)
    {
        super();
        if ( pStr == null )
        {
            append( this.nullArray );
            return;
        }
        this.chrArr = new char[ pStr.length() ];
        this.append( pStr );
    }
    /**
     * 
     * @param pLength
     */
    public void setLength(
            final int pLength)
    {
        if (pLength > this.nextPos)
        {
            expand(pLength - this.nextPos + 1);
//            while (this.pos < pLength)
//            {
//                // TODO sollte hier nicht lieber mit spaces (Leerzeichen) aufgefllt werden ?
//                this.array[this.pos] = '\0';
//                this.pos++;
//            }
            // TODO sollte hier nicht lieber mit spaces (Leerzeichen) aufgefllt werden ?
            Arrays.fill( this.chrArr, this.nextPos, pLength, '\0');
        }
        this.nextPos = pLength;
    }
    
    /**
     * 
     */
    public void fillSpacesToLength(
            final int pLength)
    {
        if (pLength > this.nextPos)
        {
            expand(pLength - this.nextPos + 1);
//            while (this.pos < pLength)
//            {
//                // TODO sollte hier nicht lieber mit spaces (Leerzeichen) aufgefllt werden ?
//                this.array[this.pos] = '\0';
//                this.pos++;
//            }
            Arrays.fill( this.chrArr, this.nextPos, pLength, ' ');
            this.nextPos = pLength;
        }
    }
    
    
    /**
     * clear buffer.
     */
    public void clear()
    {
        this.setLength( 0 );
    }
    /**
     * 
     * @param pChr
     */
    public void append(
            final char pChr )
    {
        expand(1);
        this.chrArr[this.nextPos++] = pChr;
    }
    /**
     * 
     * @param pCharArr
     */
    public void append(
            final char[] pCharArr )
    {
        if (pCharArr == null)
        {
            expand(4);
            System.arraycopy(this.nullArray, 0, this.chrArr, this.nextPos, 4);
            this.nextPos += 4;
        }
        else
        {
            int len = pCharArr.length;
            if (len > 0)
            {
                expand(len);
                System.arraycopy(pCharArr, 0, this.chrArr, this.nextPos, len);
                this.nextPos += len;
            }
        }
    }
    /**
     * Append String.
     * @param pStr
     */
    public void append(
            final String pStr)
    {
        if (pStr == null)
        {
            expand(4);
            "null".getChars(0, 4, this.chrArr, this.nextPos);
            this.nextPos += 4;
        }
        else
        {
            int len = pStr.length();
            if (len > 0)
            {
                expand(len);
                pStr.getChars(0, len, this.chrArr, this.nextPos);
                this.nextPos += len;
            }
        }
    }
    /**
     * append Object
     * @param pValue
     */
    public void append(
            final Object pValue)
    {
        this.append( String.valueOf( pValue ) );
    }
    // TODO Methode append(ReuseStringBuffer)
    /**
     * 
     */
    public String toString()
    {
        return new String(this.chrArr, 0, this.nextPos);
    }
    
    /**
     * expand size of char array.
     * 
     * @param pSize
     */
    private void expand(
            final int pSize)
    {
        if ((this.nextPos + pSize) >= this.chrArr.length)
        {
            // Vergroessere char Array
            // aber mindestens um den absoluten INCREASE_TRESHOLD oder um den relativen INCREASE_FACTOR
            final int newLength = ( this.chrArr.length + Math.max( pSize , INCREASE_TRESHOLD ) ) * INCREASE_FACTOR ;

            final char[] tmpChrArr = new char[ newLength ];
//            for (int i = 0; i < this.pos; i++)
//            {
//                tmpChrArr[i] = this.array[i];
//            }
            System.arraycopy(this.chrArr, 0, tmpChrArr, 0, this.nextPos + 1 );

            this.chrArr = tmpChrArr;
        }
    }
    /**
     * @return
     */
    public boolean isEmpty()
    {
        return this.length() < 1;
    }
    /**
     * @return
     */
    public int length()
    {
        return this.nextPos;
    }
    /**
     * Return toString or null, if buffer is empty.
     * 
     * @return
     */
    public Object toStringOrNullIfEmpty()
    {
        if ( this.isEmpty() )
        {
            return null;
        }
        return this.toString();
    }
    
    /**
     * 
     * @param pPos
     * @return
     */
    public char charAt(
            final int pPos)
    {
        if ( pPos > this.nextPos )
        {
            throw new ArrayIndexOutOfBoundsException( "pPos: " + pPos + " currPos: " + this.nextPos );
        }
        return this.chrArr[ pPos ];
    }

    /**
     * 
     * @param pPos
     * @return
     */
    public void setCharAt(
            final int pPos,
            final char pChr)
    {
        if ( pPos > this.nextPos )
        {
            throw new ArrayIndexOutOfBoundsException( "pPos: " + pPos + " currPos: " + this.nextPos );
        }
        this.chrArr[ pPos ] = pChr;
    }
    /**
     * Zusammefgen aller Elemente des bergebenen
     * StringBuffer-Arrays mit dem bergebenen Trenn-String.
     * 
     * @param pStrArr zusammzufgendes String-Array
     * @param pDelimStr Trenn-String
     * @return
     */
    public static String concat(
            final UnSynchStringBuffer[] pStrBuffArr, 
            final String pDelimStr)
    {
        final UnSynchStringBuffer concatStrBuff = new UnSynchStringBuffer( 1024 );
        //synchronized (concatStrBuff)
        {
            for (int i = 0; i < pStrBuffArr.length; i++)
            {
                final String str = pStrBuffArr[i].toString();
                concatStrBuff.append( str );
                if ( pDelimStr != null 
                        && i < pStrBuffArr.length - 1 )
                {
                    concatStrBuff.append( pDelimStr );
                }
            }
            return concatStrBuff.toString();
        }
    }
    /**
     * Get a clone.
     * 
     * @return
     */
    public UnSynchStringBuffer getClone()
    {
        final UnSynchStringBuffer clone = new UnSynchStringBuffer( 0 );
        clone.nextPos = this.nextPos;
        clone.chrArr = (char[]) this.chrArr.clone();
        return clone;
    }
    /**
     * Insert specfied character at specified position.
     * @param pPos
     * @param pC
     */
    public void insert(
            final int pPos, 
            final char pChar)
    {
        if ( pPos < 0 )
        {
            throw new IllegalArgumentException( "insert pos: " + pPos );
        }
        if ( pPos == this.nextPos )
        {
            //throw new IllegalArgumentException( "insert pos: " + pPos + " maximum possible position is " + ( this.nextPos - 1 ) );
            append( pChar );
            return;
        }
        //if ( pPos >= this.nextPos )
        if ( pPos > this.nextPos )
        {
            throw new IllegalArgumentException( "insert pos: " + pPos + " maximum possible position is " + ( this.nextPos - 1 ) );
        }
        expand( this.nextPos + 1 );
        System.arraycopy( 
                this.chrArr, 
                pPos, 
                this.chrArr, 
                pPos + 1, 
                this.nextPos - pPos );
        this.chrArr[ pPos ] = pChar;
        this.nextPos++;
    }

    /**
     * Delete character at specified position.
     * @param pPos
     */
    public void delete(
            final int pPos)
    {
        if ( pPos < 0 )
        {
            throw new IllegalArgumentException( "delete pos: " + pPos );
        }
        if ( pPos >= this.nextPos )
        {
            throw new IllegalArgumentException( "delete pos: " + pPos + " maximum possible position is " + ( this.nextPos - 1 ) );
        }
        System.arraycopy( 
                this.chrArr, 
                pPos + 1, 
                this.chrArr, 
                pPos, 
                this.nextPos - pPos - 1 );
        this.nextPos--;
    }

    
    
    /**
     * Test.
     * 
     * @param args
     */
    public static void main(String[] args)
    {
        final UnSynchStringBuffer strBuff = new UnSynchStringBuffer( "012345" );
        System.out.println( strBuff );

        strBuff.insert( 0 , 'I' );
        System.out.println( strBuff );
        
        strBuff.delete( 0 );
        System.out.println( strBuff );
        
        strBuff.insert( 6 , 'I' );
        System.out.println( strBuff );
        
        strBuff.delete( 6 );
        System.out.println( strBuff );
    }

}
