
package util.test;

import util.CollectionFormatUtil;

/**
 * Klasse zum Erzeugen von Test-Fllen.
 * 
 * Das Prinzip dieser Klasse ist an
 * {@link BigInteger} angelehnt.
 * 
 * @see Variants
 * 
 * TODO evtl. Umstellung auf Iterator-Prinzip: hasNext() und next()
 * 
 * @author Heiner Kcker
 */
public class BigVariants
{
    /**
     * Array aus {@link Variants}-Objekten, aus denen dieses Objekt aufgebaut ist.
     */
    private final Variants[] variantsArr;
    
    /**
     * berlauf-Merker
     */
    private boolean carry;

    /**
     * Constructor.
     *
     * @param pVariantsArr
     */
    public BigVariants(
            final Variants[] pVariantsArr)
    {
        // avoid null
        for (int i = 0; i < pVariantsArr.length; i++)
        {
            if ( pVariantsArr[ i ] == null )
            {
                throw new NullPointerException( "pVariantsArr[ " + i + "] is null" );
            }
        }
        this.variantsArr = pVariantsArr;
    }

    /**
     * @return the carry
     */
    public boolean carry()
    {
        return this.carry;
    }

    /**
     * Clear this.
     */
    public void clear()
    {
        for (int i = 0; i < this.variantsArr.length; i++)
        {
            final Variants variants = this.variantsArr[ i ];
            variants.clear();
        }
        this.carry = false;
    }

    /**
     * Um eins erhhen.
     */
    public void increment()
    {
        if ( this.carry )
        {
            throw new ArithmeticException( "value is overflow" );
        }
        for (int i = 0; i < this.variantsArr.length; i++)
        {
            final Variants variants = this.variantsArr[ i ];
            variants.increment();
            if ( ! variants.carry() )
            {
                // alle Stellen erledigt
                return;
            }
            // Zurcksetzen und bertrag auf die nchste Stelle
            variants.clear();
        }
        // berlauf
        this.carry = true;
    }
    
    /**
     * Get complex value (zweidimensional).
     * @return
     */
    public Object[] getValueArr()
    {
        if ( this.carry )
        {
            throw new ArithmeticException( "value is overflow" );
        }
        final Object[] retValueArr = new Object[ this.variantsArr.length ];
        for (int i = 0; i < retValueArr.length; i++)
        {
            retValueArr[i] = this.variantsArr[ i ].getValue();
        }
        return retValueArr;
    }

    /**
     * @param args
     */
    public static void main(String[] args)
    {
        BigVariants bigVariants = new BigVariants( 
                new Variants[] {
                        new Variants( new Object[] { Boolean.FALSE , Boolean.TRUE } )
                }
                 );
        while ( ! bigVariants.carry() )
        {
            System.out.println( CollectionFormatUtil.toStr( bigVariants.getValueArr() ) );
            bigVariants.increment();
        }
        
        System.out.println();

        bigVariants = new BigVariants( 
                new Variants[] {
                        new Variants( new Object[] { "Tintenstrahldrucker" , "Laserdrucker" } ),
                        new Variants( new Object[][] { 
                                { "Schrift nicht kursiv" , "Schrift kursiv" },
                                { "Schrift nicht fett" , "Schrift fett" } }
                        ) 
                    }
                ) ;
        while ( ! bigVariants.carry() )
        {
            System.out.println( CollectionFormatUtil.toStr( bigVariants.getValueArr() ) );
            bigVariants.increment();
        }
        
        System.out.println();

    }

}
