package util.test;

import java.util.Arrays;


/**
 * Klasse zum Erzeugen von Test-Fllen.
 * 
 * Das Prinzip dieser Klasse ist an
 * {@link BigInteger} angelehnt.
 * 
 * Beispiel:
 * 
 * new Variants( new Object[] { Boolean.FALSE , Boolean.TRUE } );
 * (einfachster Fall, binr, hier lieber {@link BigBinary} verwenden)
 * 
 * new Variants( new Object[] { "Schrift nicht kursiv" , "Schrift kursiv" } );
 * (eindimensional, zwei mgliche Kombinationen)
 * 
 * new Variants( new Object[][] { 
 *     { "Schrift nicht kursiv" , "Schrift kursiv" },
 *     { "Schrift nicht fett" , "Schrift fett" } } );
 * (zweidimensional, vier mgliche Kombinationen)
 * 
 * Eventuell auch mit null-Werten:
 * new Variants( new Object[][] { 
 *     { null , "Schrift nicht kursiv" , "Schrift kursiv" },
 *     { null , "Schrift nicht fett" , "Schrift fett" } } );
 * (zweidimensional, sechs mgliche Kombinationen)
 * 
 * Der Name 'Variants' steht einfach fr mehrere
 * mgliche alternative Werte. Evtl. wre der
 * Name 'Enum' besser.
 * 
 * TODO evtl. Umstellung auf Iterator-Prinzip: hasNext() und next()
 * 
 * TODO Evtl Umbenennen in Enum und zustzlich noch eine Klasse Range (von, bis, Schrittweite) machen
 * 
 * @author Heiner Kcker
 */
public class Variants
{
    /**
     * Die mglichen Werte-Kombinationen
     */
    final Object[][] valueRangeArrArr;
    
    /**
     * Die Index-Nummern aktuellen Werte
     */
    final int[] currValueIndexArr;
              
    boolean carry;

    /**
     * Constructor zweidimensionaler Wertebereich
     *
     * @param pValueRangeArrArr
     */
    public Variants(
            final Object[][] pValueRangeArrArr)
    {
        this.valueRangeArrArr = pValueRangeArrArr;
        this.currValueIndexArr = new int[ pValueRangeArrArr.length ];
    }

    /**
     * Constructor eindimensionaler Werte-Bereich.
     *
     * @param pValueRangeArr
     */
    public Variants(
            final Object[] pValueRangeArr )
    {
        this.valueRangeArrArr = new Object[][]{ pValueRangeArr };
        this.currValueIndexArr = new int[ 1 ];
    }

    /**
     * @return
     * TODO dies ist keine OOP, wegen dem erforderlichen if-else, evtl. aufteilen in drei Klassen AbstractVariants, OneDimensionalVariants, TwoDimensionalVariants
     */
    public boolean isOneDimensional()
    {
        return this.currValueIndexArr.length < 2;
    }

    /**
     * @return
     * TODO dies ist keine OOP, wegen dem erforderlichen if-else, evtl. aufteilen in drei Klassen AbstractVariants, OneDimensionalVariants, TwoDimensionalVariants
     */
    public boolean isTwoDimensional()
    {
        return this.currValueIndexArr.length > 1;
    }
    
    /**
     * @return the carry
     */
    public boolean carry()
    {
        return this.carry;
    }
    
    /**
     * Zum nchsten Wert weiterschalten.
     */
    public void increment()
    {
        if ( this.carry )
        {
            throw new ArithmeticException( "value is overflow" );
        }
        for (int i = 0; i < this.currValueIndexArr.length; i++)
        {
            if ( this.currValueIndexArr[ i ] < this.valueRangeArrArr[ i ].length - 1 )
            {
                this.currValueIndexArr[ i ]++;
                // alle Stellen erledigt
                return;
            }
            else
            {
                // Zurcksetzen und bertrag auf die nchste Stelle
                this.currValueIndexArr[ i ] = 0;
            }
        }
        // berlauf
        this.carry = true;
    }

    /**
     * Get simple value (eindimensional).
     * @return
     */
    public Object getValue()
    {
        if ( this.carry )
        {
            throw new ArithmeticException( "value is overflow" );
        }
        if ( isTwoDimensional() )
        {
            //throw new IllegalStateException( "dies ist ein komplexes Variants: " + this );
            return this.getValueArr();
            
        }
        return this.valueRangeArrArr[ 0 ][ this.currValueIndexArr[ 0 ] ];
    }

    /**
     * Get complex value (zweidimensional).
     * @return
     */
    public Object[] getValueArr()
    {
        if ( isOneDimensional() )
        {
            //throw new IllegalStateException( "dies ist ein simples Variants: " + this );
            return new Object[] { this.getValue() };
        }
        if ( this.carry )
        {
            throw new ArithmeticException( "value is overflow" );
        }
        final Object[] retValueArr = new Object[ this.currValueIndexArr.length ];
        for (int i = 0; i < retValueArr.length; i++)
        {
            retValueArr[i] = this.valueRangeArrArr[ i ][ this.currValueIndexArr[ i ] ];
        }
        return retValueArr;
    }

    /**
     * Clear this.
     */
    public void clear()
    {
        Arrays.fill( this.currValueIndexArr , 0 );
        this.carry = false;
    }

    /**
     * Test.
     * @param args
     */
    public static void main(String[] args)
    {
        Variants variants = 
            new Variants( 
                    new Object[] { 
                            Boolean.FALSE , 
                            Boolean.TRUE } );
        while ( ! variants.carry() )
        {
            System.out.println( variants.getValue() );
            variants.increment();
        }

        System.out.println();

        variants = 
            new Variants( 
                    new Object[] { 
                            "Schrift nicht kursiv" , 
                            "Schrift kursiv" } );
        while ( ! variants.carry() )
        {
            System.out.println( variants.getValue() );
            variants.increment();
        }
        
        System.out.println();

        variants = 
            new Variants( new Object[][] { 
                { "Schrift nicht kursiv" , "Schrift kursiv" },
                { "Schrift nicht fett" , "Schrift fett" } } );
        while ( ! variants.carry() )
        {
            System.out.println( Arrays.asList( variants.getValueArr() ) );
            variants.increment();
        }
        
        System.out.println();

        variants = 
            new Variants( new Object[][] { 
                { null , "Schrift nicht kursiv" , "Schrift kursiv" },
                { null , "Schrift nicht fett" , "Schrift fett" } } );
        while ( ! variants.carry() )
        {
            System.out.println( Arrays.asList( variants.getValueArr() ) );
            variants.increment();
        }
    }
}
